package edu.washington.cs.oneswarm.ui.gwt.client.newui;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.HorizontalSplitPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmDialogBox;
import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmRPCClient;
import edu.washington.cs.oneswarm.ui.gwt.rpc.TorrentInfo;

public class SwarmDetailsDialog extends OneSwarmDialogBox implements ChangeHandler {
	public static final int WIDTH = 600;
	public static final int HEIGHT = 400;

	private TorrentInfo[] mSwarms = null;
	private CheckBox shareWithFriendsCheckbox;
	private CheckBox shareWithPublic;
	private ListBox swarmList;

	private final Set<Integer> changed = new HashSet<Integer>();

	public SwarmDetailsDialog(EntireUIRoot inRoot, TorrentInfo[] inSwarms) {
		super(false);

		setText("Swarm permissions");

		mSwarms = inSwarms;

		HorizontalSplitPanel mainPanel = new HorizontalSplitPanel();

		swarmList = new ListBox();
		mainPanel.add(swarmList);

		for (TorrentInfo t : inSwarms) {
			swarmList.addItem(t.getName());
		}
		swarmList.setVisibleItemCount(inSwarms.length);

		swarmList.addChangeHandler(this);

		VerticalPanel rhs = new VerticalPanel();

		mainPanel.add(rhs);

		rhs.add(new Label("Permissions:"));
		HorizontalPanel hp = new HorizontalPanel();
		shareWithFriendsCheckbox = new CheckBox("Share with friends");
		shareWithPublic = new CheckBox("Share publicly");
		hp.add(shareWithFriendsCheckbox);
		hp.add(shareWithPublic);

		shareWithFriendsCheckbox.addClickHandler(this);
		shareWithPublic.addClickHandler(this);

		rhs.add(hp);

		rhs.add(new Label("Changes will be applied when this window is closed."));

		rhs.setWidth("100%");
		hp.setWidth("100%");

		mainPanel.setSplitPosition("60%");

		mainPanel.setWidth(WIDTH + "px");
		mainPanel.setHeight(HEIGHT + "px");

		setWidget(mainPanel);

		onChange(null);
	}

	public void hide() {
		if (changed.size() == 0) {
			super.hide();
			return;
		}

		System.out.println("synchronizing swarm details");

		final List<TorrentInfo> changedList = new ArrayList<TorrentInfo>();
		for (Integer i : changed) {
			changedList.add(mSwarms[i]);
		}

		OneSwarmRPCClient.getService().applySwarmPermissionChanges(OneSwarmRPCClient.getSessionID(), changedList, new AsyncCallback<Void>() {
			public void onFailure(Throwable caught) {
				caught.printStackTrace();
			}

			public void onSuccess(Void result) {
				System.out.println("sync'd list of: " + changedList.size());
			}
		});

		super.hide();
	}

	public void onClick(ClickEvent event) {
		boolean touched = false;

		if (event.getSource().equals(shareWithFriendsCheckbox)) {
			mSwarms[swarmList.getSelectedIndex()].setShareWithFriends(shareWithFriendsCheckbox.getValue());
			touched = true;
		} else if (event.getSource().equals(shareWithPublic)) {
			mSwarms[swarmList.getSelectedIndex()].setSharePublic(shareWithPublic.getValue());
			touched = true;
		} else {
			super.onClick(event);
		}

		if (touched) {
			changed.add(swarmList.getSelectedIndex());
		}
	}

	public void onChange(ChangeEvent sender) {
		if (swarmList.getSelectedIndex() == -1) {
			shareWithFriendsCheckbox.setEnabled(false);
			shareWithPublic.setEnabled(false);
		} else {
			shareWithFriendsCheckbox.setEnabled(true);
			shareWithPublic.setEnabled(true);

			TorrentInfo info = mSwarms[swarmList.getSelectedIndex()];

			shareWithFriendsCheckbox.setValue(info.getShareWithFriends());
			shareWithPublic.setValue(info.getSharePublic());
		}
	}
}
