package edu.washington.cs.oneswarm.ui.gwt.client.newui.settings;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DecoratedTabPanel;
import com.google.gwt.user.client.ui.DisclosurePanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmDialogBox;
import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmGWT;
import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmRPCClient;
import edu.washington.cs.oneswarm.ui.gwt.client.filebrowser.TorrentDownloaderDialog;
import edu.washington.cs.oneswarm.ui.gwt.client.newui.EntireUIRoot;
import edu.washington.cs.oneswarm.ui.gwt.client.newui.OneSwarmCss;
import edu.washington.cs.oneswarm.ui.gwt.client.newui.SwarmsBrowser;

public class SettingsDialog extends OneSwarmDialogBox {
	public static final String CSS_HRULE = "os-hrule";

	private static final int WIDTH = 450;
	private static final int HEIGHT = 200;

	PortsSettingsPanel ports_settings = new PortsSettingsPanel();
	SaveLocationPanel save_loc = new SaveLocationPanel();
	SpeedLimitsPanel speed_limits = new SpeedLimitsPanel();
	MagicDirectorySettingsPanel magic_dirs = new MagicDirectorySettingsPanel();
	UISettingsPanel ui_settings = new UISettingsPanel();
	TagsSettingsPanel tags_settings = new TagsSettingsPanel();

	List<SettingsPanel> settingsPanels = new ArrayList<SettingsPanel>();
	PrivacySettingsPanel privacySettingsPanel = new PrivacySettingsPanel();
	RemoteAccessPanel remoteAccessPanel = new RemoteAccessPanel();
	// DataUsage dataUsagePanel = new DataUsage();
	CommunityServersSettingsPanel communityPanel = new CommunityServersSettingsPanel();

	boolean cancel = false;

	Button refreshAssociationsButton = new Button("Refresh file associations");

	private SwarmsBrowser mSwarmsBrowser;

	DecoratedTabPanel mTabs;

	public SettingsDialog(EntireUIRoot inRoot, SwarmsBrowser swarmsBrowser, int selectedTab) {
		super(false, true, true);

		mSwarmsBrowser = swarmsBrowser;

		Label selectLabel = new Label(msg.settings_dialog_msg());
		if (OneSwarmGWT.isRemoteAccess()) {
			selectLabel.setText(msg.settings_dialog_msg_remote_access());
		}

		mTabs = new DecoratedTabPanel();
		mTabs.addStyleName(TorrentDownloaderDialog.CSS_F2F_TABS);
		final VerticalPanel filesTab = new VerticalPanel();
		// windows
		OneSwarmRPCClient.getService().getPlatform(OneSwarmRPCClient.getSessionID(), new AsyncCallback<String>() {
			public void onFailure(Throwable caught) {
				caught.printStackTrace();
			}

			public void onSuccess(String result) {
				if (result.equals("windows")) {
					refreshAssociationsButton.addStyleName(OneSwarmCss.SMALL_BUTTON);
					refreshAssociationsButton.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							OneSwarmRPCClient.getService().refreshFileAssociations(OneSwarmRPCClient.getSessionID(), new AsyncCallback<Void>() {
								public void onFailure(Throwable caught) {
									caught.printStackTrace();
								}

								public void onSuccess(Void result) {
									System.out.println("refresh file associations: success");
								}
							});
						}
					});
					filesTab.add(refreshAssociationsButton);
				}
			}
		});
		filesTab.add(save_loc);
		filesTab.add(magic_dirs);
		mTabs.add(filesTab, msg.settings_tab_files());

		VerticalPanel networkTab = new VerticalPanel();
		networkTab.add(speed_limits);
		networkTab.add(ports_settings);
		
		DisclosurePanel communityDisclosurePanel = new DisclosurePanel(msg.settings_net_community_servers(), true);
		communityDisclosurePanel.add(communityPanel);
		networkTab.add(communityDisclosurePanel);
		
		if (OneSwarmGWT.isRemoteAccess() == false) {
			DisclosurePanel remoteAccessDisclosurePanel = new DisclosurePanel(msg.settings_net_remote_access(), true);
			remoteAccessDisclosurePanel.add(remoteAccessPanel);
			networkTab.add(remoteAccessDisclosurePanel);
		}
		
		mTabs.add(networkTab, msg.settings_tab_network());

		VerticalPanel uiTab = new VerticalPanel();
		uiTab.add(ui_settings);
		DisclosurePanel tagsDisclosurePanel = new DisclosurePanel(msg.settings_interface_tags(), true);
		tagsDisclosurePanel.add(tags_settings);
		uiTab.add(tagsDisclosurePanel);
		mTabs.add(uiTab, msg.settings_tab_interface());

		VerticalPanel privacyTab = new VerticalPanel();
		privacyTab.add(privacySettingsPanel);
		mTabs.add(privacyTab, msg.settings_tab_privacy());

		mTabs.selectTab(selectedTab);
		mTabs.setWidth("100%");
		mTabs.setHeight("100%");

		selectLabel.addStyleName(CSS_DIALOG_HEADER);
		selectLabel.setWidth(WIDTH + "px");

		settingsPanels.add(save_loc);
		settingsPanels.add(magic_dirs);
		settingsPanels.add(speed_limits);
		settingsPanels.add(ports_settings);
		settingsPanels.add(ui_settings);
		settingsPanels.add(tags_settings);
		settingsPanels.add(privacySettingsPanel);
		if (OneSwarmGWT.isRemoteAccess() == false) {
			settingsPanels.add(remoteAccessPanel);
		}
		// settingsPanels.add(dataUsagePanel);
		settingsPanels.add(communityPanel);

		setText(msg.settings_dialog_header());

		// mUIRoot = inRoot;

		VerticalPanel main_vp = new VerticalPanel();

		main_vp.add(selectLabel);
		main_vp.add(mTabs);
		main_vp.setCellVerticalAlignment(mTabs, VerticalPanel.ALIGN_TOP);

		// main_vp.add(ports_settings);
		// main_vp.add(save_loc);
		// main_vp.add(speed_limits);
		// main_vp.add(magic_dirs);
		// main_vp.add(ui_settings);
		// DisclosurePanel privacyDisclosurePanel = new
		// DisclosurePanel("Privacy settings", false);
		// privacyDisclosurePanel.add(privacySettingsPanel);
		// main_vp.add(privacyDisclosurePanel);
		//
		// DisclosurePanel tagsDisclosurePanel = new DisclosurePanel("Tags",
		// false);
		// tagsDisclosurePanel.add(tags_settings);
		// main_vp.add(tagsDisclosurePanel);
		//		
		// DisclosurePanel remoteAccessDisclosurePanel = new
		// DisclosurePanel("Remote access", false);
		// remoteAccessDisclosurePanel.add(remoteAccessPanel);
		// main_vp.add(remoteAccessDisclosurePanel);
		//		
		// // DisclosurePanel dataUsageDisclosurePanel = new
		// DisclosurePanel("Data usage", false);
		// // dataUsageDisclosurePanel.add(dataUsagePanel);
		// // main_vp.add(dataUsageDisclosurePanel);
		//		
		// DisclosurePanel communityDisclosurePanel = new
		// DisclosurePanel("Community servers", false);
		// communityDisclosurePanel.add(communityPanel);
		// main_vp.add(communityDisclosurePanel);

		// com.google.gwt.user.client.ui.Widget hrule = new SimplePanel();
		// hrule.addStyleName(CSS_HRULE);
		// main_vp.add(hrule);
		HorizontalPanel revert_defaults_hp = new HorizontalPanel();
		HorizontalPanel lhs = new HorizontalPanel();
		lhs.setSpacing(3);
		HorizontalPanel rhs = new HorizontalPanel();
		Button saveButton = new Button(msg.button_save());
		Button cancelButton = new Button(msg.button_cancel());

		cancelButton.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				cancel = true;
				hide();
			}
		});

		saveButton.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				hide();
			}
		});

		rhs.add(cancelButton);
		rhs.add(saveButton);
		rhs.setSpacing(3);
		revert_defaults_hp.add(lhs);
		revert_defaults_hp.add(rhs);
		revert_defaults_hp.setCellHorizontalAlignment(lhs, HorizontalPanel.ALIGN_LEFT);
		revert_defaults_hp.setCellHorizontalAlignment(rhs, HorizontalPanel.ALIGN_RIGHT);
		revert_defaults_hp.setWidth("100%");

		main_vp.add(revert_defaults_hp);

		main_vp.setWidth(WIDTH + "px");
		main_vp.setHeight(HEIGHT + "px");

		setWidget(main_vp);
	}

	public void hide() {
		if (!cancel) {
			for (SettingsPanel p : settingsPanels) {
				String error = p.validData();
				if (error != null) {
					Window.alert(error);
					return;
				}
			}

			for (SettingsPanel p : settingsPanels) {
				if (p.isReadyToSave()) {

					/**
					 * Not allowed, so just skip.
					 */
					if (OneSwarmGWT.isRemoteAccess() && p instanceof RemoteAccessPanel) {
						continue;
					}

					p.sync();
				}
			}

			/**
			 * This has to come _after_ all the settings have actually
			 * synchronized, otherwise we'll miss any changes. Although this is
			 * a hack, synchronizing after a second works.
			 */
			(new Timer() {
				public void run() {
					mSwarmsBrowser.sync_settings();
				}
			}).schedule(1000);
		}
		super.hide();
	}

}
